from PyQt4 import QtGui, QtCore

class ControlWidget(QtGui.QWidget):
	"""
	A widget containing controllers for a single parameter
	Layout:
	|---------------|
	| label | field |
	|---------------|
	|     slider    |
	|---------------|
	"""
	valueChanged = QtCore.pyqtSignal(float)

	def __init__(self, parent, name, value, minimum, maximum, ticks=100):
		super(ControlWidget, self).__init__(parent)
		self._minimum = minimum
		self._resolution = float(maximum-minimum)/ticks
		# self.value = value

		self._field = QtGui.QDoubleSpinBox(self)
		self._slider = QtGui.QSlider(QtCore.Qt.Horizontal, self)
		self._field.setRange(minimum, maximum)
		self._field.setSingleStep(self._resolution)
		self._field.valueChanged.connect(self.field_updated)

		self._slider.setRange(0, ticks)	# slider uses integers only
		self._slider.valueChanged.connect(self.slider_updated)
		self._slider.setValue(self._convert_to_slider(value))

		# upper horizontal boxes
		hbox = QtGui.QHBoxLayout()
		hbox.setContentsMargins(0,0,0,0)
		hbox.addWidget(QtGui.QLabel(name))
		hbox.addWidget(self._field)
		hcontainer = QtGui.QWidget()
		hcontainer.setLayout(hbox)

		# vertical layout
		vbox = QtGui.QVBoxLayout()
		vbox.setSpacing(0)
		vbox.setContentsMargins(0,0,0,0)
		vbox.addWidget(hcontainer)
		vbox.addWidget(self._slider)
		self.setLayout(vbox)

	# def set_value(self, value):
	# 	self.value = value

	def _convert_to_slider(self, value):
		"""
		Slider accepts integers only, so floats have to be converted
		"""
		return int((value - self._minimum)/self._resolution)

	def _convert_from_slider(self, scaled):
		"""
		Slider accepts integers only, so floats have to be converted
		"""
		return self._minimum + scaled * self._resolution

	def slider_updated(self, scaled):
		"""
		Called when the slider is updated. Updates the field and callback
		Slider gives the value as an integer, so it has to be scaled
		"""
		value = self._convert_from_slider(scaled)
		self._field.setValue(value)
		self.valueChanged.emit(value)

	def field_updated(self, value):
		"""
		Called when the field is updated. Updates the slider and callback
		Field gives the values straight as a float,
		but it has to be converted to integer for slider
		"""
		scaled = self._convert_to_slider(value)
		self._slider.setValue(scaled)
		self.valueChanged.emit(value)


	#######################
	#### Getters & setters:

	def get_minimum(self):
		return self._minimum

	def set_minimum(self, minimum):
		self._minimum = float(minimum)
	minimum = property(get_minimum, set_minimum)

	def get_resolution(self):
		return self._resolution

	def set_resolution(self, resolution):
		if resolution < 0:
			raise ValueError("Resolution must be > 0")
		self._resolution = float(resolution)
	resolution = property(get_resolution, set_resolution)
