class Boid(object):
	"""
	Boid class is a simple vehicle model
	"""
	def __init__(self, position=complex(0,0), velocity=complex(0, 0), mass=1, orientation=complex(1, 0),
				max_force=400, min_speed=80, max_speed=110):
		self.mass = mass
		self.position = position
		self.velocity = velocity
		self.max_force = max_force
		self.min_speed = min_speed
		self.max_speed = max_speed
		self.orientation = orientation

	def move(self, dt, force=complex(0,0)):
		"""
		Moves this bird according to the given time step
		and applies the given force to boid during this update
		dt is in seconds
		"""
		# simple Euler integration:
		if abs(force) > self.max_force:
			force = force/abs(force) * self.max_force
		acceleration = force/self.mass
		self.velocity += acceleration * dt
		self.limit_speed()
		self.position += self.velocity * dt
		# in this case orientation is always parallel to velocity
		speed = abs(self.velocity)
		if speed > 0:
			self.orientation = self.velocity/speed

	def limit_speed(self):
		"""
		Limits the speed of the boid so it doesn't go too fast nor too slow
		"""
		speed = abs(self.velocity)
		if speed > self.max_speed:
			self.velocity = self.velocity/speed * self.max_speed
		elif speed < self.min_speed:
			self.velocity = self.velocity/speed * self.min_speed


	#######################
	#### Getters & setters:

	def get_position(self):
		return self._position

	def set_position(self, position):
		self._position = complex(position)
	position = property(get_position, set_position)

	def get_velocity(self):
		return self._velocity

	def set_velocity(self, velocity):
		self._velocity = complex(velocity)
	velocity = property(get_velocity, set_velocity)

	def get_mass(self):
		return self._mass

	def set_mass(self, mass):
		if mass <= 0:
			raise ValueError("mass of boid should be > 0")
		self._mass = float(mass)
	mass = property(get_mass, set_mass)

	def get_orientation(self):
		return self._orientation

	def set_orientation(self, orientation):
		length = abs(orientation)
		if length > 0:
			self._orientation = complex(orientation)/length	# normalize
		else:
			self._orientation = complex(1, 0)	# right
	orientation = property(get_orientation, set_orientation)

	def get_min_speed(self):
		return self._min_speed

	def set_min_speed(self, min_speed):
		self._min_speed = float(min_speed)
	min_speed = property(get_min_speed, set_min_speed)

	def get_max_speed(self):
		return self._max_speed

	def set_max_speed(self, max_speed):
		self._max_speed = max_speed
	max_speed = property(get_max_speed, set_max_speed)

	def get_max_force(self):
		return self._max_force

	def set_max_force(self, max_force):
		if max_force < 0:
			raise ValueError("maximum force must be >= 0")
		self._max_force = max_force
	max_force = property(get_max_force, set_max_force)
