import time
from PyQt4 import QtCore
from world import World

class FlockSimulation(QtCore.QThread):
	"""
	FlockSimulation drives the entire simulation
	"""
	def __init__(self, parent, width, height):
		super(FlockSimulation, self).__init__(parent)
		self.parent = parent
		self._world = World(width, height)
		self.init_slots()
		self.start()

	def run(self):
		"""
		The main loop function
		"""
		old_time = time.time() # in seconds
		new_time = old_time
		while True:
			#update timer
			dt = new_time - old_time
			old_time = new_time
			new_time = time.time()

			self.world.update(dt)
			self.notify_ui()
			time.sleep(0.02)
			# if dt < 0.01:
			#	time.sleep(0.05)

	def notify_ui(self):
		"""
		Notifies UI about world updates
		"""
		self.emit(QtCore.SIGNAL('boids(PyQt_PyObject)'),
			self.world.get_boids())

	def init_slots(self):
		"""
		Connects slots used in thread communication
		"""
		self.connect(self.parent, QtCore.SIGNAL('boid_count(int)'),
			self.world.set_boid_count)
		self.connect(self.parent, QtCore.SIGNAL('neighbor_radius(float)'),
			self.world.set_neighbor_radius)
		self.connect(self.parent, QtCore.SIGNAL('neighbor_angle(float)'),
			self.world.set_neighbor_angle)
		self.connect(self.parent, QtCore.SIGNAL('separation_weight(float)'),
			self.world.set_separation_weight)
		self.connect(self.parent, QtCore.SIGNAL('alignment_weight(float)'),
			self.world.set_alignment_weight)
		self.connect(self.parent, QtCore.SIGNAL('cohesion_weight(float)'),
			self.world.set_cohesion_weight)
		self.connect(self.parent, QtCore.SIGNAL('resize(int,int)'),
			self.world.resize)
		# TODO: add all necessary slots


	#######################
	#### Getters & setters:

	def get_world(self):
		return self._world
	world = property(get_world)
